from Globals import DTMLFile, default__class_init__
from Products.PlugIns import PlugInFinder, MakePICBase
from Products.ZPatterns.Specialists import Specialist

class ExtendedSpecialist(Specialist):
    """ Specialist with methods commonly used for adding and deleting
        instances.
    """

    meta_type = 'ExtendedSpecialist'

    index_html = DTMLFile('dtml/index', globals())

    selectTag = DTMLFile('dtml/selectTag', globals())

    selectInstance = DTMLFile('dtml/selectInstance', globals())


    def addInstance(self, REQUEST=None, **kw):
        """ Create a new instance ... """
        if REQUEST is None:
            props={}
        else: props=REQUEST
        if kw:
            for name, value in kw.items():
                props[name]=value

        id = self.rackList[0].__of__(self).getNewID()
        ni = self.newItem(id)
        for name, ps in ni.propertysheets.items():
            if name == 'webdav': continue
            for id in ps.propertyIds():
                if props.has_key(id):
                    setattr(ni, id, props[id])

        if REQUEST:
            try: url = REQUEST['DestinationURL']
            except: url = REQUEST['URL1']
            REQUEST.RESPONSE.redirect(url)
        else:
            return ni


    def deleteInstances(self, ids=[], REQUEST=None):
        """ Delete instances """
        if not ids:
            ids = REQUEST.get('ids', [])
        for id in ids:
            article = self.getItem(id)
            article.manage_delete()

        if REQUEST:
            try: url = REQUEST['DestinationURL']
            except: url = REQUEST['URL1']
            REQUEST.RESPONSE.redirect(url)
        

    def getItems(self, **kw):
        """ Delegate list retrieval to Rack """
        properties = {}
        if kw:
            for key, value in kw.items():
                properties[key] = value
        return self.rackList[0].__of__(self).getItems(properties)

    def managedMetaType(self):
        """ Return metatype of ZClass managed by Specialist """
        zclass = self.rackList[0].__of__(self)._zclass
        meta_type = zclass.split('/')[-1]
        return meta_type

    def setControl(self, namespace, control_name=None, control_value=None):
        ''' Setup the tag/value pair for a generic select tag.

            The 'control_name' and initially selected 'control_value' of
            the select tag is obtained as follows:

                Control Name

                - if 'namespace.has_key('control_name')', the tag is
                  named 'namespace['control_name']', overriding the
                  default name if any was passed in as parameter.
                  (Values set in '<dtml-let>'s are in the namespace.)

                - if the 'REQUEST.has_key('control_name')', the tag is
                  named 'REQUEST['control_name']', overriding the
                  default name if any was passed in as parameter.

                - otherwise, the tag is named 'control_name' if it was
                  passed as parameter. If nothing was passed, the tag is
                  named 'select'.

                Control Value

                - if 'namespace.has_key('control_value')', the initially
                  selected value of the tag is set to
                  'namespace['control_value']', overriding the default
                  value if any was passed in as parameter, or available
                  in the 'context'. 

                - if the 'REQUEST' contains a key with the name of the
                  tag, the initially selected value of the tag is set to
                  the value of the key.

                - otherwise, the tag name is looked for in the context,
                  and the initially selected value of the tag is set to
                  its value.

        '''
        REQUEST = self.REQUEST
        context = self.aq_parent

        ## Get value for 'control_name'
        # Override the "sticky" value?
        if namespace.has_key('control_name'):
            control_name = namespace['control_name']
        elif REQUEST.has_key('control_name'): # Look on REQUEST first
            control_name = REQUEST['control_name']
        elif not control_name: # If it was not passed in, check 'context'
            # If no overrides, go find the "sticky" value:
            if hasattr(context, 'control_name'): # Look in the namespace
                control_name = getattr(context, 'control_name')
            else:
                control_name = 'select'

        ## Get value for 'control_value'
        # Override the "sticky" value?
        if namespace.has_key('control_value'):
            control_value = namespace['control_value']
        elif REQUEST.has_key('control_value'):
            control_value = REQUEST['control_value'] 
        elif not control_value: 
            # If no overrides, go find the "sticky" value:
            if hasattr(context, 'control_value'):
                control_value = getattr(context, 'control_value')
            else: 
                if REQUEST.has_key(control_name):
                    control_value = REQUEST[control_name] 
                elif hasattr(context, control_name): 
                    control_value = getattr(context, control_name)
                else:
                    control_value = None

        return control_name, control_value


default__class_init__(ExtendedSpecialist); MakePICBase(ExtendedSpecialist)


def genericAddForm(klass, globalDict=None):

    """Return a customized DTMLFile of the generic plugin Add form"""
    
    className = klass.__name__
    formName = 'manage_add%sForm' % className
    
    f = DTMLFile(
        'dtml/genericAddForm', globals(),
        RacksMetaTypes = PlugInFinder(Specialist.RacksGroup),
        title_        = klass.meta_type,
        constructor_  = ('manage_add%s' % className),
        __name__      = formName
    )

    if globalDict is not None: globalDict[formName] = f
    return f


_defaultConstructor = '''
def manage_add%(name)s(self, id, title='', rack='',
        connection_id='', zclass='', REQUEST=None):
    """Add %(meta_type)s"""
    ob = %(name)s(id, title)
    self._setObject(id, ob)
    ob = ob.__of__(self)
    
    if rack:
        if rack =='SQLRack':
            ob.RacksGroup._constructPlugIn(rack, id='defaultRack',
                zclass=zclass, connection_id=connection_id)
        else:
            ob.RacksGroup._constructPlugIn(rack, id='defaultRack')
    
    if REQUEST: return self.manage_main(self,REQUEST,update_menu=1)
'''

def simpleConstructor(klass, globalDict):
    name=klass.__name__; meta_type = klass.meta_type
    exec _defaultConstructor % locals() in globalDict
    return globalDict['manage_add%s' % name]


def defaultConstructors(klass, globalDict):
    return (
        genericAddForm(klass, globalDict),
        simpleConstructor(klass, globalDict)
    )

def initialize(context):

    context.registerPIContainerBase(ExtendedSpecialist)

    context.registerClass(
        ExtendedSpecialist,
        permission = 'Add Specialists',
        constructors = defaultConstructors(ExtendedSpecialist, globals()),
        icon = 'www/specialist.gif',
    )
