"""Customizable SkinScript that come from the filesystem."""

from string import strip, split
from os import path, stat
import new

import Globals
from AccessControl import ClassSecurityInfo, getSecurityManager

from Products.ZPatterns.Providers import NullProvider
from Products.ZPatterns.PlugIns import PlugIn
from Products.ZPatterns.SkinScript.Compiler import compile
from Products.ZPatterns.SkinScript.Methods import SSMethod
from Products.ZPatterns.PlugIns import PlugIn, defaultConstructors
from Globals import DTMLFile, default__class_init__

from Products.CMFCore.utils import _dtmldir
from Products.CMFCore.CMFCorePermissions import ViewManagementScreens, View, FTPAccess
from Products.CMFCore.DirectoryView import registerFileExtension, registerMetaType, expandpath
from Products.CMFCore.FSObject import FSObject

import zLOG

class FSSkinScript (FSObject, PlugIn, NullProvider):
    """FSSkinScripts act like SkinScripts but are not directly
    modifiable from the management interface."""

    __plugin_kind__ = "Attribute Provider"
    meta_type = "Filesystem SkinScript Method"

    text = ''
    _providers = ()

    manage_options=(
            (
            {'label':'Customize',
             'action':'manage_main',
             'help': ('ZPatterns','SkinScriptSyntax.stx')},
            )
        )

    # Use declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    security.declareProtected(View, 'index_html',)
    # Prevent the bindings from being edited TTW

    security.declareProtected(ViewManagementScreens, 'manage_main')
    manage_main = DTMLFile('dtml/custss', globals())

    def _createZODBClone(self):
        """Create a ZODB (editable) equivalent of this object."""
        obj = SSMethod(self.getId())
        text = self.read()
        obj.manage_edit('', self.read())
        return obj

    def _readFile(self, reparse):
        """Read the data from the filesystem.
        
        Read the file (indicated by exandpath(self._filepath), and parse the
        data if necessary.
        """
        fp = expandpath(self._filepath)
        file = open(fp, 'r')
        try: data = file.read()
        finally: file.close()
        self.text = data

    def _validateProxy(self, roles=None):
        pass

    def _propertiesChanged(self):
        providers = self._providers = compile(self.text)
        
        for p in providers:
            #p._proxy_roles = self._proxy_roles
            p.id = '%s, line %d' % (self.id,p.lineno); p.title = self.title


        # We can either be in the context of the Specialist or the
        # DirectoryView
        if self.aq_parent.meta_type == 'SkinnableSpecialist':
            specialist = self.aq_parent
        else:
            specialist = self.aq_parent.aq_parent.aq_parent

        specialist.manage_refreshPlugIns()
            

    def _getProviders(self):
        return self._providers


    def PrincipiaSearchSource(self):
        "Support for searching - the document's contents are searched."
        return self.text


    def manage_FTPget(self):
        "Get source for FTP download"
        self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/plain')
        return self.read()

    def addToContainer(self):
        self._setup()
        container = self.aq_parent.aq_parent.aq_parent
        if self.__plugin_kind__ and hasattr(container,'_addPlugIn'):
            container._addPlugIn(self)

    def read(self):
        self._updateFromFS()
        self.addToContainer()
        self._propertiesChanged()
        return self.text

    def get_size(self): return len(self.text)

    getSize = get_size


default__class_init__(FSSkinScript)


def initialize(context):
    context.registerPlugInClass(
        FSSkinScript,
        constructors = defaultConstructors(FSSkinScript, globals()),
        icon = 'www/SkinScriptMethod.gif'
    )


registerFileExtension('ss', FSSkinScript)
registerMetaType('SkinScript Method', FSSkinScript)
