import string, copy
from Products.ZPatterns.Rack import Rack
from Products.PlugIns import MakePICBase
from Globals import DTMLFile, default__class_init__
from ComputedAttribute import ComputedAttribute
from Products.ZPatterns.SkinScript.Methods import SSMethod

sqlMap = {'string': 'varchar(20)',
          'date': 'date'}

zsqlMap = {'string':'string', 
           'int':'int',
           'date':'string',
           'lines': 'string',
           'text': 'string'}

class SQLRack(Rack):
    """ Generates ZSQLMethods and SkinScript for ZClass upon
        initialization.
    """

    manage_storageForm = DTMLFile('dtml/storageForm', globals())

    meta_type = "SQLRack"

    def getItems(self, properties):
        return self.sqlSelect(properties)

    def getNewID(self):
        sql = 'SELECT max(oid) FROM %s' % self.aq_parent.managedMetaType()

        self._setSQLMethod('sqlGetNewID', '', '', sql)

        r = self.sqlGetNewID()
        if r.tuples()[0] == (None,):
            return 1
        else:
            id = int(r[0][0]) + 1
            return id


    def manage_setStorage(self, zclass=None, connection_id='', 
        storage=None, initialize_tables=0, 
        load_attrib=None, REQUEST=None):
        """
        Generate sql methods and skinscript for class used as storage
        """

        self.primaryKey = load_attrib
        use_attrib = "YES"
        Rack.manage_setStorage(self, zclass, storage, use_attrib,
            load_attrib)

        self.connection_id = connection_id

        classname = string.split(zclass, '/')[-1]
        if classname != getattr(self, 'classname', ''):
            self.classname = classname
            self._columns = None

        self.createSQLLever()
        self.selectSQLLever()
        self.insertSQLLever()
        self.updateSQLLever()
        self.deleteSQLLever()
        self.skinScriptLever()
        
        if initialize_tables:
            try:
                self.sqlCreate()
            except:
                pass

        if REQUEST is not None:
            return self.manage_storageForm(self, REQUEST,
                manage_tabs_message='Updated storage settings.')


    def createSQLLever(self, sqlName='sqlCreate'):
        """ Generate SQL for table creation """

        columns, idref_names, idrefs_names = self._getSchema()

        columndefs = ['%s %s NOT NULL' % (self.primaryKey, 
                                          sqlMap.get('int', 'int'))]
        for name, type in columns:
            sql_type = sqlMap.get(type, type)
            columndefs.append('%s %s' % (name, sql_type))
        columnsString = ',\n    '.join(columndefs)
        sql = '''\
create table %s (
    %s
)
''' % (self.classname, columnsString)

        for idrefs_classname in idrefs_names:
            table_name = idrefs_classname + 'sFor' + self.classname
            sql_type = sqlMap.get('string', 'varchar(20)')
            sql += '''
<dtml-var sql_delimiter> 

create table %s (
    %sID %s,
    %sID %s
)
''' % (table_name, self.classname, sql_type, idrefs_classname, sql_type)

        args = [sqlName, '', self.connection_id, '', sql]
        self._setZSQLMethod(sqlName, args)


    def selectSQLLever(self, sqlName='sqlSelect'):
        """ Generate SQL select statement """

        columns, idref_names, idrefs_names = self._getSchema()

        argumentList = []
        whereList = []
        columns.insert(0, (self.primaryKey, 'int'))

        whereClause = """
    <dtml-sqlgroup where>
    %s
    </dtml-sqlgroup>
        """

        simpleGroup = """
    <dtml-sqlgroup>
    <dtml-if %s>
    <dtml-sqltest %s type="%s">
    </dtml-if>
    </dtml-sqlgroup>
    """

        # generate 'select' SQL statement
        for name, type in columns:
            zsql_type = zsqlMap.get(type, type)
            argumentList.append('%s=""' % name)
            whereList.append( simpleGroup % (name, name, zsql_type))

        argumentString = '\n'.join(argumentList)
        columnString = ',\n    '.join([name for name, type in columns])
        whereTemplate = '<dtml-and>'.join(whereList)
        whereString = whereClause % whereTemplate

        sql = """\
select
    %s
from %s
%s
    """% (columnString, self.classname, whereString)

        # set 'select' ZSQLMethods
        args = [sqlName, '', self.connection_id, argumentString, sql]
        self._setZSQLMethod(sqlName, args)

        # set 'select' ZSQLMethods and Python Scripts for mapping tables
        for idrefs_classname in idrefs_names:
            table_name = idrefs_classname + 'sFor' + self.classname
            where = '<dtml-sqltest %sID type="string">'% self.classname
            sql = '''\
select %sID
from %s
where %s
    ''' % (idrefs_classname, table_name, where)

            sqlName = 'sqlSelect%sIDs'%idrefs_classname

            args = [sqlName, '', self.connection_id, self.classname+'ID', sql]
            self._setZSQLMethod(sqlName, args)

            scriptName = 'get%sIDs'%idrefs_classname
            params = self.classname + 'ID'
            body = '''\
records = container.sqlSelect%sIDs(%sID=%sID)
result = []
for record in records:
    result.append(record.%sID)

return result
    '''%(idrefs_classname, self.classname, self.classname, idrefs_classname)

            self._setPythonScript(scriptName, params, body)


    def insertSQLLever(self, sqlName='sqlInsert'):
        """ Generate SQL insert statement """

        columns, idref_names, idrefs_names = self._getSchema()

        # generate 'insert' SQL statement
        argumentList = ['%s=""' % self.primaryKey]
        columndefs = ['<dtml-sqlvar %s type="%s">' % (
            self.primaryKey, zsqlMap.get('int', 'int'))]
        for name, type in columns:
            zsql_type = zsqlMap.get(type, type)
            argumentList.append('%s=""' % name)
            columndefs.append('<dtml-sqlvar %s type="%s">' % (name, zsql_type))

        argumentString = string.join(argumentList, '\n')
        columnsString = ',\n    '.join(columndefs)
        sql = '''\
insert into %s 
values (
    %s
)
    ''' % (self.classname, columnsString)

        # set 'insert' ZSQLMethods
        args = [sqlName, '', self.connection_id, argumentString, sql]
        self._setZSQLMethod(sqlName, args)

        # set 'insert' ZSQLMethods and Python Scripts for mapping tables
        for idrefs_classname in idrefs_names:
            table_name = idrefs_classname + 'sFor' + self.classname
            argumentString = '%sID=""\n%sID=""' % (self.classname, idrefs_classname)
            values = ['<dtml-sqlvar %sID type="string">' % self.classname]
            values.append('<dtml-sqlvar %sID type="string">' % idrefs_classname)
            valueString = ',\n    '.join(values)
            sql = '''\
insert into %s
values (
    %s
)
    ''' % (table_name, valueString)

            sqlName = 'sqlInsert%sIDs' % idrefs_classname

            args = [sqlName, '', self.connection_id, argumentString, sql]
            self._setZSQLMethod(sqlName, args)

            scriptName = 'insert%sIDs'%idrefs_classname
            params = '%sID, %sIDs' % (self.classname, idrefs_classname)
            body = '''\
for id in %sIDs:
    container.sqlInsert%sIDs(%sID=%sID, %sID=id)
    '''%(idrefs_classname, idrefs_classname, self.classname, self.classname, idrefs_classname)

            self._setPythonScript(scriptName, params, body)


    def updateSQLLever(self, sqlName='sqlUpdate'):
        """ Generate SQL update statement """

        columns, idref_names, idrefs_names = self._getSchema()

        classname = self.classname

        # generate 'update' SQL statement
        argumentList = ['%s=""' % self.primaryKey]
        columndefs = []
        for name, type in columns:
            zsql_type = zsqlMap.get(type, type)
            argumentList.append('%s=""' % name)
            columndefs.append('%s = <dtml-sqlvar %s type="%s">' % (
                name, name, zsql_type))

        argumentString = string.join(argumentList, '\n')
        columnsString = ',\n    '.join(columndefs)
        sql = '''\
update %s set
    %s
where
    oid = <dtml-sqlvar oid type="int">
''' % (classname, columnsString)

        # set 'update' ZSQLMethods
        args = [sqlName, '', self.connection_id, argumentString, sql]
        self._setZSQLMethod(sqlName, args)

        # set 'update' Python Scripts for mapping tables
        for idrefs_classname in idrefs_names:
            scriptName = 'update%sIDs'%idrefs_classname
            params = '%sID, %sIDs' % (classname, idrefs_classname)
            body = '''\
container.sqlDelete%(idrefs_classname)sIDs(%(classname)sID=%(classname)sID)

for id in %(idrefs_classname)sIDs:
    container.sqlInsert%(idrefs_classname)sIDs(%(classname)sID=%(classname)sID, %(idrefs_classname)sID=id)
'''%locals()

            self._setPythonScript(scriptName, params, body)

    def deleteSQLLever(self, sqlName='sqlDelete'):
        """ Generate SQL delete statement """

        columns, idref_names, idrefs_names = self._getSchema()

        # generate 'delete' SQL statement
        argumentString = 'oid=""'
        sql = '''\
delete from %s 
where
    <dtml-sqltest %s type="int">
''' % (self.classname, self.primaryKey)

        # set 'delete' ZSQLMethod
        args = [sqlName, '', self.connection_id, argumentString, sql]
        self._setZSQLMethod(sqlName, args)

        # generate 'delete' SQL for mapping tables
        for idrefs_classname in idrefs_names:
            sqlName = 'sqlDelete%sIDs' % idrefs_classname
            argumentString = '%sID=""' % self.classname
            table_name = idrefs_classname + 'sFor' + self.classname
            valuesString = '<dtml-sqltest %sID type="string">' % self.classname
            sql = '''
delete from %s
where
    %s
''' % (table_name, valuesString)

            args = [sqlName, '', self.connection_id, argumentString, sql]
            self._setZSQLMethod(sqlName, args)


    def skinScriptLever(self):
        """ Generate skinscript """

        columns, idref_names, idrefs_names = self._getSchema()

        classname = self.classname
        skinName = classname + 'SkinScript'

        computeList = ["%s = %s" % (self.primaryKey, self.primaryKey)]
        saveList = ["%s = self.id" % self.primaryKey]
        storeList = []

        for name, type in columns:
            saveList.append("%s = self.%s" % (name, name))
            computeList.append("%s = %s" % (name, name))
            storeList.append(name)

        idrefs_AddedStrings = []
        idrefs_ChangedStrings = []
        idrefs_ComputeStrings = []
        idref_ComputeStrings = []
        idrefs_DeletedStrings = []

        # generate skinscript for idrefs
        for idrefs_classname in idrefs_names:
            idrefs_AddedStrings.append('''
WHEN OBJECT ADDED CALL insert%(idrefs_classname)sIDs(
    self.id, self.%(idrefs_classname)sIDs)''' % locals()
            )

            idrefs_ChangedStrings.append('''
WHEN OBJECT CHANGED STORE %(idrefs_classname)sIDs
USING update%(idrefs_classname)sIDs(
    self.id, self.%(idrefs_classname)sIDs)''' % locals()
            )

            idrefs_ComputeStrings.append('''
WITH get%(idrefs_classname)sIDs(self.id) COMPUTE 
    %(idrefs_classname)sIDs = RESULT or []''' % locals()
            )

            idrefs_DeletedStrings.append('''
WHEN OBJECT DELETED CALL sqlDelete%(idrefs_classname)sIDs(
    %(classname)sID=self.id)''' % locals()
            )

        # generate skinscript of idref
        for idref_classname in idref_names:
            idref_ComputeStrings.append('''
WITH %(idref_classname)ss.getItem(self.%(idref_classname)sID) COMPUTE
    %(idref_classname)s = RESULT or NOT_FOUND''' % locals()
            )

        # generate skinscript for object adding
        addedString = """
WHEN OBJECT ADDED CALL sqlInsert(
    %s
    )
""" % (',\n    '.join(saveList))
        if idrefs_AddedStrings:
            addedString += """\n%s""" % ( '\n\n'.join(idrefs_AddedStrings) )

        # generate skinscript for object changing
        changedString = """
WHEN OBJECT CHANGED STORE
    %s
USING sqlUpdate(
    %s
    )
""" % ( ',\n    '.join(storeList), 
        ',\n    '.join(saveList) )
        if idrefs_ChangedStrings:
            changedString += """\n%s""" % ( '\n\n'.join(idrefs_ChangedStrings) )

        # generate skinscript for object retrieval
        computeString = """
WITH QUERY sqlSelect(%s=self.id) COMPUTE
    %s
""" % ( self.primaryKey, ',\n    '.join(computeList) )
        if idrefs_ComputeStrings:
            computeString += """\n%s""" % ( '\n\n'.join(idrefs_ComputeStrings) )
        if idref_ComputeStrings:
            computeString += """\n%s""" % ( '\n\n'.join(idref_ComputeStrings) )

        # generate skinscript for object deleting
        deletedString = """
WHEN OBJECT DELETED CALL sqlDelete(%s=self.id)
""" % ( self.primaryKey )
        if idrefs_DeletedStrings:
            deletedString += """\n%s""" % ( '\n\n'.join(idrefs_DeletedStrings) )

        if not hasattr(self, skinName):
            fc = self.manage_addProduct['ZPatterns']
            fc.manage_addSSMethod(skinName, '')

        script = getattr(self, skinName)
        text= '%s\n\n%s\n\n%s\n\n%s' % ( addedString,
                                         deletedString,
                                         changedString,
                                         computeString 
                                        )
        try:
            script.manage_edit( title='', text=text)
        except:
            raise text
    

    # Private methods
    def _setZSQLMethod(self, method_name, args):
        if not hasattr(self, method_name):
            method = getattr(self.manage_addProduct['ZSQLMethods'],
                            'manage_addZSQLMethod')
            apply(method, args)
        else:
            method = getattr(self, method_name).manage_edit
            apply(method, args[1:])


    def _setPythonScript(self, method_name, params, body):
        if not hasattr(self, method_name):
            method = getattr(self.manage_addProduct['PythonScripts'],
                            'manage_addPythonScript')
            apply(method, [method_name])

        method = getattr(self, method_name).ZPythonScript_edit
        apply(method, [params, body])


    def _getPropertyMap(self):
        columns = [] 
        idrefs_names = []
        idref_names = []

        # import pdb; pdb.set_trace()
        # is it a file system product
        _zclass = self._unifiedZClassRegistry()[self._zclass][1]
        if _zclass.__module__ == 'ZClasses.ZClass':
            SkinZISS = _zclass.propertysheets.common
            sheet_dicts = SkinZISS._objects
            properties = []
            for dict in sheet_dicts:
                if dict['meta_type'] == \
                    'DataSkin Attribute Property Sheet':
                    sheet_id = dict['id']
                    properties += SkinZISS[sheet_id]._properties
            if not properties:
                raise "ZClass has no DataSkin Attribute Property Sheet"
        else:
            properties = _zclass._zclass_._properties

        for item in properties:
            if item['type'] == 'lines' and item['id'].find('IDs') != -1:
                idrefs_classname = item['id'][:-3]
                idrefs_names.append(idrefs_classname)
            else:
                if item['id'].find('ID') and item['id'][-2:] == 'ID':
                    idref_classname = item['id'][:-2]
                    idref_names.append(idref_classname)
                columns.append((item['id'], item['type']))

        return columns, idref_names, idrefs_names

    def _getSchema(self):
        """ Get the class schema """

        if not self._columns:
            self._columns, self._idref_names, self._idrefs_names = \
                self._getPropertyMap()

        columns = copy.deepcopy(self._columns)
        idref_names = copy.deepcopy(self._idref_names)
        idrefs_names = copy.deepcopy(self._idrefs_names)
        return columns, idref_names, idrefs_names


default__class_init__(SQLRack); MakePICBase(SQLRack)

manage_addSQLRackForm = DTMLFile('dtml/addSQLRack', globals())


def manage_addSQLRack(self, id, title='', zclass='', connection_id='',
        REQUEST=None):
    """Add a SQL Rack"""
    ob = SQLRack(id, title)
    self._setObject(id, ob)
    ob = ob.__of__(self)
    ob.loadAttrib = "oid"
    ob._columns = None

    message = "Added SQLRack"
    if REQUEST is not None:
        group = self.Destination()._findGroupFor(ob)
        return group.manage_main(group, REQUEST, update_menu=1,
            manage_tabs_message=message, URL='%s/%s/manage_workspace'%(
                REQUEST.URL1, group.id))



def initialize(context):

    context.registerPlugInClass(
        SQLRack,
        permission='Add SQL Racks',
        constructors=(manage_addSQLRackForm,
                      manage_addSQLRack),
        icon='www/sqlrack.gif',
        )

    context.registerPIContainerBase(SQLRack)


