from Globals import DTMLFile, default__class_init__
from Products.PlugIns import PlugInFinder, MakePICBase
from Products.ZPatterns.Specialists import Specialist
from Products.CMFCore.Skinnable import SkinnableObjectManager
from Products.CMFCore.utils import getToolByName

class SkinnableSpecialist(Specialist, SkinnableObjectManager):
    """ Specialist with methods commonly used for adding and deleting
        instances.
    """

    meta_type = 'SkinnableSpecialist'

    __getattr__ = SkinnableObjectManager.__getattr__
    __of__ = SkinnableObjectManager.__of__
    _checkId = SkinnableObjectManager._checkId

    def getSkinsFolderName(self):
        # Not implemented.
        return 'portal_skins'

    def setup(self, globals_dict):
        """ Add skins to specialist """
        addCMFCoreTool = self.manage_addProduct['CMFCore'].manage_addTool
        addCMFCoreTool('CMF Skins Tool', None)

        from Products.CMFCore.DirectoryView import addDirectoryViews
        ps = getToolByName(self, 'portal_skins')
        addDirectoryViews(ps, 'skins', globals_dict)
        ps.manage_addProduct['OFSP'].manage_addFolder(id='custom')
        ps.addSkinSelection('Basic', 'custom, default', make_default=1)
        self.setupCurrentSkin()


default__class_init__(SkinnableSpecialist); MakePICBase(SkinnableSpecialist)


def genericAddForm(klass, globalDict=None):

    """Return a customized DTMLFile of the generic plugin Add form"""
    
    className = klass.__name__
    formName = 'manage_add%sForm' % className
    
    f = DTMLFile(
        'dtml/genericAddForm', globals(),
        RacksMetaTypes = PlugInFinder(Specialist.RacksGroup),
        title_        = klass.meta_type,
        constructor_  = ('manage_add%s' % className),
        __name__      = formName
    )

    if globalDict is not None: globalDict[formName] = f
    return f


_defaultConstructor = '''
def manage_add%(name)s(self, id, title='', rack='',
        connection_id='', zclass='', REQUEST=None):
    """Add %(meta_type)s"""
    ob = %(name)s(id, title)
    self._setObject(id, ob)
    ob = ob.__of__(self)
    ob.setup(globals())
    
    if rack:
        if rack =='SQLRack':
            ob.RacksGroup._constructPlugIn(rack, id='defaultRack',
                zclass=zclass, connection_id=connection_id)
        else:
            ob.RacksGroup._constructPlugIn(rack, id='defaultRack')
    
    if REQUEST: return self.manage_main(self,REQUEST,update_menu=1)
'''

def simpleConstructor(klass, globalDict):
    name=klass.__name__; meta_type = klass.meta_type
    exec _defaultConstructor % locals() in globalDict
    return globalDict['manage_add%s' % name]


def defaultConstructors(klass, globalDict):
    return (
        genericAddForm(klass, globalDict),
        simpleConstructor(klass, globalDict)
    )

def initialize(context):

    context.registerPIContainerBase(SkinnableSpecialist)

    context.registerClass(
        SkinnableSpecialist,
        permission = 'Add Specialists',
        constructors = defaultConstructors(SkinnableSpecialist, globals()),
        icon = 'www/specialist.gif',
    )

